package xyz.lychee.lagfixer;

import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.bukkit.Bukkit;
import xyz.lychee.lagfixer.libs.annotations.NotNull;

public class ColoredLogger extends Logger {
   private static final String COLOR_PATTERN = "\u001b[38;5;%dm";
   private static final String FORMAT_PATTERN = "\u001b[%dm";

   public ColoredLogger() {
      super(convertStringMessage("&e⚡"), (String)null);
      this.setParent(Bukkit.getLogger());
      this.setLevel(Level.ALL);
   }

   private static String convertStringMessage(String message) {
      if (message != null && !message.isEmpty()) {
         String messageCopy = String.copyValueOf(message.toCharArray()) + ColoredLogger.ConsoleColor.RESET.ansiColor;

         String result;
         ConsoleColor color;
         for(Matcher matcher = Pattern.compile(String.format("(%c[0-9a-fk-or])(?!.*\u0001)", '&')).matcher(message); matcher.find(); messageCopy = messageCopy.replace(result, color.getAnsiColor())) {
            result = matcher.group(1);
            color = ColoredLogger.ConsoleColor.getColorByCode(result.charAt(1));
         }

         return messageCopy;
      } else {
         return message;
      }
   }

   public void log(@NotNull LogRecord logRecord) {
      logRecord.setMessage(convertStringMessage(logRecord.getMessage()));
      super.log(logRecord);
   }

   private static enum ConsoleColor {
      BLACK('0', "\u001b[38;5;%dm", 0),
      DARK_GREEN('2', "\u001b[38;5;%dm", 2),
      DARK_RED('4', "\u001b[38;5;%dm", 1),
      GOLD('6', "\u001b[38;5;%dm", 172),
      DARK_GREY('8', "\u001b[38;5;%dm", 8),
      GREEN('a', "\u001b[38;5;%dm", 10),
      RED('c', "\u001b[38;5;%dm", 9),
      YELLOW('e', "\u001b[38;5;%dm", 11),
      DARK_BLUE('1', "\u001b[38;5;%dm", 4),
      DARK_AQUA('3', "\u001b[38;5;%dm", 30),
      DARK_PURPLE('5', "\u001b[38;5;%dm", 54),
      GRAY('7', "\u001b[38;5;%dm", 246),
      BLUE('9', "\u001b[38;5;%dm", 4),
      AQUA('b', "\u001b[38;5;%dm", 51),
      LIGHT_PURPLE('d', "\u001b[38;5;%dm", 13),
      WHITE('f', "\u001b[38;5;%dm", 15),
      STRIKETHROUGH('m', "\u001b[%dm", 9),
      ITALIC('o', "\u001b[%dm", 3),
      BOLD('l', "\u001b[%dm", 1),
      UNDERLINE('n', "\u001b[%dm", 4),
      RESET('r', "\u001b[%dm", 0);

      private final char bukkitColor;
      private final String ansiColor;

      private ConsoleColor(char bukkitColor, String pattern, int ansiCode) {
         this.bukkitColor = bukkitColor;
         this.ansiColor = String.format(pattern, ansiCode);
      }

      public static ConsoleColor getColorByCode(char code) {
         ConsoleColor[] var1 = values();
         int var2 = var1.length;

         for(int var3 = 0; var3 < var2; ++var3) {
            ConsoleColor color = var1[var3];
            if (color.bukkitColor == code) {
               return color;
            }
         }

         throw new IllegalArgumentException("Color with code " + code + " does not exists");
      }

      public char getBukkitColor() {
         return this.bukkitColor;
      }

      public String getAnsiColor() {
         return this.ansiColor;
      }

      // $FF: synthetic method
      private static ConsoleColor[] $values() {
         return new ConsoleColor[]{BLACK, DARK_GREEN, DARK_RED, GOLD, DARK_GREY, GREEN, RED, YELLOW, DARK_BLUE, DARK_AQUA, DARK_PURPLE, GRAY, BLUE, AQUA, LIGHT_PURPLE, WHITE, STRIKETHROUGH, ITALIC, BOLD, UNDERLINE, RESET};
      }
   }
}
